﻿using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;

namespace Example_ConnectionComponent1.Handlers
{
    public class PlateHandler
    {
        private readonly ComponentInstanceContext m_context = null;
        private readonly ParamsHandler m_paramsHandler = null;
        //
        private readonly PlateProperties plate_props = new PlateProperties();  //多边形板参数
        int plate_angle_type = new int();     //倒角类型
        double plate_angle_x = new double();  //倒角x方向大小
        double plate_angle_y = new double();  //倒角y方向大小

        public PlateHandler(ComponentInstanceContext context, ParamsHandler paramsHandler)
        {
            m_context = context;
            m_paramsHandler = paramsHandler;
        }
        public void Execute()
        {
            //初始化
            InitParams();
            //创建多边形板示例代码(加劲肋几乎都是由多边形板来创建，故以创建加劲肋为例):
            {
                //1.设置控制点点集合或包围面集合(上部加劲肋由控制点创建,下部加劲肋由包围面创建)
                //控制点
                Vec3[] points = Set_PlatePoint(m_context.SubPlane_top, m_context.MainPart, m_context.MainPlanes);
                //包围面
                Plane top_plane = m_context.SubPlane_bottom.Clone();
                Plane bottom_plane = top_plane.Move(plate_props.Thickness).Clone();
                Plane[] side_plane = Set_PlateSidePlane(m_context.MainPart, m_context.MainPlanes);
                //2.创建,设置参数
                SteelPlate top_plate = m_context.Comp.GetOrCreateSteelPlate("top_plate", points, plate_props.Thickness, LoongCAD.STC.Define.ObjectSide.SIDE_MIDDLE);
                SteelPlate bottom_plate = m_context.Comp.GetOrCreateSteelPlate("bottom_plate", top_plane, bottom_plane, side_plane);
                plate_props.ApplyToPart(top_plate); plate_props.ApplyToPart(bottom_plate);
                //3.设置折角(只设置了top_plane的其中一个角，其它角同理设置)
                ChamferCtrlPoint chamferCtrlPoint = top_plate.CtrlPoints[0] as ChamferCtrlPoint;
                switch (plate_angle_type)
                {
                    case 2:
                        chamferCtrlPoint.CutType = Define.CornerCutType.CORNER_CUT_LINE;
                        chamferCtrlPoint.CutX = plate_angle_x;
                        chamferCtrlPoint.CutY = plate_angle_y;
                        chamferCtrlPoint.Refresh();
                        break;
                    case 3:
                        chamferCtrlPoint.CutType = Define.CornerCutType.CORNER_CUT_ROUND;
                        chamferCtrlPoint.CutR = plate_angle_x;
                        chamferCtrlPoint.Refresh();
                        break;
                    case 4:
                        chamferCtrlPoint.CutType = Define.CornerCutType.CORNER_CUT_INV_ROUND;
                        chamferCtrlPoint.CutR = plate_angle_x;
                        chamferCtrlPoint.Refresh();
                        break;
                    default:
                        break;
                }
            }
            //创建多边形板切割(被切割的是直梁)
            {
                //1.设置点(也可用面创建)
                Vec3[] points = new Vec3[4];
                points[0] = m_context.Beam_Part.CtrlPoints[0].WorldPos + m_context.Normal_right * 20;
                points[1] = points[0] - m_context.Normal_right * 40;
                points[2] = points[1] - m_context.Normal_top * 200;
                points[3] = points[2] + m_context.Normal_right * 40;
                //2.创建多边形板切割体
                m_context.Comp.CreatePlateCutter(m_context.Beam_Part, points, plate_props.Thickness, Define.ObjectSide.SIDE_RIGHT, true);
                //3.设置倒角(和多边形板设置倒角是一样的)
            }
        }
        private void InitParams()
        {
            plate_props.SetProps(m_paramsHandler.GetDObjectValue(ParamNames.PLATE_PROPS));
            plate_angle_type = m_paramsHandler.GetIntValue(ParamNames.PLATE_ANGLE);
            plate_angle_x = m_paramsHandler.GetDoubleValue(ParamNames.PLATE_ANGLE_X);
            plate_angle_y = m_paramsHandler.GetDoubleValue(ParamNames.PLATE_ANGLE_Y);
        }
        /// <summary>
        /// 创建加劲肋，返回控制点
        /// </summary>
        /// <param name="plane">加劲肋创建的中心面(位置)</param>
        /// <param name="steelBeam">要创建加劲肋的梁</param>
        /// <param name="beamPlanes">该梁的零件包围面</param>
        /// <returns></returns>
        private Vec3[] Set_PlatePoint(Plane plane, Beam steelBeam, BeamPlanes beamPlanes)
        {
            //创建加劲肋一定要判断,腹板或翼缘存不存在，否则会奔溃
            Vec3[] CtrlPoints = new Vec3[4];
            if (beamPlanes.TopFlange == null)
                return null;
            {
                CtrlPoints[0] = plane.Intersect(beamPlanes.FrontBound, beamPlanes.TopFlange.Move(-steelBeam.Profile.ShapeFlangeThickness1));
                CtrlPoints[1] = plane.Intersect(beamPlanes.FrontWeb, beamPlanes.TopFlange.Move(-steelBeam.Profile.ShapeFlangeThickness1));

                if (beamPlanes.BottomFlange != null)
                {
                    CtrlPoints[2] = plane.Intersect(beamPlanes.FrontWeb, beamPlanes.BottomFlange.Move(-steelBeam.Profile.ShapeFlangeThickness2));
                    CtrlPoints[3] = plane.Intersect(beamPlanes.FrontBound, beamPlanes.BottomFlange.Move(-steelBeam.Profile.ShapeFlangeThickness2));
                }
                else
                {
                    CtrlPoints[2] = plane.Intersect(beamPlanes.FrontWeb, beamPlanes.BottomBound.Move(-steelBeam.Profile.ShapeFlangeThickness2));
                    CtrlPoints[3] = plane.Intersect(beamPlanes.FrontBound, beamPlanes.BottomBound.Move(-steelBeam.Profile.ShapeFlangeThickness2));

                }
            }
            return CtrlPoints;
        }

        /// <summary>
        /// 创建加劲肋，返回包围面
        /// </summary>
        /// <param name="steelBeam">要创建加劲肋的梁</param>
        /// <param name="beamPlanes">该梁的零件包围面</param>
        /// <returns></returns>
        private Plane[] Set_PlateSidePlane(Beam steelBeam, BeamPlanes beamPlanes)
        {
            //创建加劲肋一定要判断,腹板或翼缘存不存在，否则会奔溃
            Plane[] side_plane = new Plane[4];
            if (beamPlanes.TopFlange == null)
                return null;
            {
                side_plane[0] = beamPlanes.TopFlange.Move(-steelBeam.Profile.ShapeFlangeThickness1).Clone();
                side_plane[1] = beamPlanes.FrontBound.Clone();
                if (beamPlanes.BottomFlange != null)
                {
                    side_plane[2] = beamPlanes.BottomFlange.Move(-steelBeam.Profile.ShapeFlangeThickness2).Clone();

                }
                else
                {
                    side_plane[2] = beamPlanes.BottomBound.Move(-steelBeam.Profile.ShapeFlangeThickness2).Clone();
                }
                side_plane[3] = beamPlanes.FrontWeb.Clone();
            }
            return side_plane;
        }
    }
}
